/*******************************************************************************
***        module: can.c
***            by: Jack Eberson
***      compiler: MikroC PRO for ARM V6.0.0
***      hardware: STM32F746ZG
***   description: Mikromedia plus STM32F746ZG CAN bus module routines
***
***   Select the appropriate CAN speed by selecting the appropriate macro.
*******************************************************************************/

#include "types.h"
#include "mmp_leddemo_helpers.h"
#include "circularbuffer.h"

// Un-comment the CAN bus speed to use
#define CAN_BAUDRATE_250KB
//#define CAN_BAUDRATE_1000KB

uint32_t Can_Init_Flags;
uint8_t Can_Send_Flags, Can_Rcv_Flags, Can_Fltr_Flags;

// CAN baudrate: 1 MBytes/s (APB1 clock: 54MHz)
#ifdef CAN_BAUDRATE_1000KB
const unsigned int SJW = 1;
const unsigned int BRP = 6;
const unsigned int PHSEG1 = 1;
const unsigned int PHSEG2 = 2;
const unsigned int PROPSEG = 5;
#endif

// CAN baudrate: 250 KBytes/s (APB1 clock: 54MHz)
#ifdef CAN_BAUDRATE_250KB
const unsigned int SJW = 3;
const unsigned int BRP = 12;
const unsigned int PHSEG1 = 6;
const unsigned int PHSEG2 = 3;
const unsigned int PROPSEG = 8;
#endif

// Setup the MCU's CAN1 controller
void Init_CAN()
{
   Can_Init_Flags = 0;
   Can_Send_Flags = 0;
   Can_Rcv_Flags  = 0;
   Can_Fltr_Flags = 0;

   Can_Send_Flags = _CAN_TX_XTD_FRAME & _CAN_TX_NO_RTR_FRAME;

   Can_Init_Flags = _CAN_CONFIG_AUTOMATIC_RETRANSMISSION &
                    _CAN_CONFIG_RX_FIFO_NOT_LOCKED_ON_OVERRUN &
                    _CAN_CONFIG_TIME_TRIGGERED_MODE_DISABLED &
                    _CAN_CONFIG_TX_FIFO_PRIORITY_BY_IDINTIFIER &
                    _CAN_CONFIG_WAKE_UP;

   // Set CAN speed.
   CAN1InitializeAdvanced(SJW, BRP, PHSEG1, PHSEG2, PROPSEG, Can_Init_Flags, &_GPIO_MODULE_CAN1_PD01);
   
   // CAN controller 'initialization' modus
   CAN1SetOperationMode(_CAN_OperatingMode_Initialization);
   
   // Filter: ON, filter: ID, extended ID's
   Can_Fltr_Flags = _CAN_FILTER_ENABLED & _CAN_FILTER_ID_MASK_MODE & _CAN_FILTER_XTD_MSG;
   
   // ID range: 0xA000 - 0xAFFF
   CANSetFilterScale32(0, Can_Fltr_Flags, 0xA000, 0x1FFFF000);
   
   // CAN controller 'normal' modus
   CAN1SetOperationMode(_CAN_OperatingMode_Normal);
   
   // Init interrupt.
   NVIC_IntEnable(IVT_INT_CAN1_RX0);
 
   // Set FMPIE1 bit in the CAN1_IER register to enable the interrupt. "NVIC_IntEnable" does not do this!
   CAN1_IER.FMPIE0 = 1;
}

// Test CAN handler
void WriteCAN_Message()
{
   CANmsg_t CANmsg;
   
   // Send a CAN message (0xAF00). Message has no specific meaning...
   CANmsg.cmsgID      = 0xAF00;
   CANmsg.cmsgDLC     = 3;
   CANmsg.cmsgData[0] = BlinkingSpeedRed.speed;
   CANmsg.cmsgData[1] = BlinkingSpeedGreen.speed;
   CANmsg.cmsgData[2] = BlinkingSpeedBlue.speed;
   CAN1Write(CANmsg.cmsgID, CANmsg.cmsgData, CANmsg.cmsgDLC, Can_Send_Flags);
   
   // Short blink of the display CAN TX led.
   T100ms[_T100ms_CAN_TxIndicator] = 2;
}

// CAN1 interrupt service routine
void CAN_interrupt() iv IVT_INT_CAN1_RX0
{
    CANmsg_t CANmsg;
    uint16_t status;

    // Read CAN messag from FIFO 0
    status = CAN1Read(0, &CANmsg.cmsgID, &CANmsg.cmsgData, &CANmsg.cmsgDLC, &Can_Rcv_Flags);

    // Push the valid message in the buffer.
    if (status != 0)
    {
      ringBufS_push(&ringBufferCAN, &CANmsg);
    }
    
    // Short blink of the display CAN RX led.
    T100ms[_T100ms_CAN_RxIndicator] = 2;
}