/*******************************************************************************
***        module: circularbuffer.c
***            by: Jack Eberson
***      compiler: MikroC PRO for ARM V6.0.0
***      hardware: Mikromedia plus with STM32F746ZG and shield
***   description: circular buffer for the CAN messages.
***                This circular buffer software has been found on the internet.
*******************************************************************************/
#include "circularbuffer.h"

// CAN message circular buffer.
// The circular buffer handler variable (ringBufferCAN) will keep track of the 
// administration of this buffer.

// The circular buffer
ringBufS_t ringBufferCAN;
CANmsg_t CANmsg;

// modulo increase function
uint16_t modulo_inc (uint16_t value, uint16_t modulus)
{
    unsigned int my_value = value + 1;
    if (my_value >= modulus)
    {
      my_value = 0;
    }
    return (my_value);
}

// Init the ringbuffer
void ringBufS_init (ringBufS_t *_this)
{
    /*****
      The following clears:
        -> buf
        -> head
        -> tail
        -> count
      and sets head = tail
    ***/
    memset (_this, 0, sizeof (*_this));
}

// Check is the ring buffer is empty.
uint16_t ringBufS_empty (ringBufS_t *_this)
{
    return (0==_this->count);
}

// Check is the ring buffer is full.
uint16_t ringBufS_full (ringBufS_t *_this)
{
    return (_this->count >= CANBUFFERLENGTH);
}

// Get an element from the ring buffer.
int ringBufS_pop (ringBufS_t *_this, CANmsg_t *canMessage)
{   
    uint8_t i;

    if (_this->count > 0)
    {
      // Copy CAN message from pointed circular buffer element into the element 'canMessage'
      canMessage->cmsgID = _this->CANbuffer[_this->tail].cmsgID;
      canMessage->cmsgDLC = _this->CANbuffer[_this->tail].cmsgDLC;
      for (i=0;i<_this->CANbuffer[_this->tail].cmsgDLC;++i)
      {
         canMessage->cmsgData[i] = _this->CANbuffer[_this->tail].cmsgData[i];
      }
      _this->tail = modulo_inc (_this->tail, CANBUFFERLENGTH);
      --_this->count;
      
      return (0);
    }
    else
    {
      return (-1);
    }
}

// Put an element in the ring buffer.
int ringBufS_push (ringBufS_t *_this,  CANmsg_t *canMessage)
{
    uint8_t i;
    
    if (_this->count < CANBUFFERLENGTH)
    {
      _this->CANbuffer[_this->head].cmsgID = canMessage->cmsgID;
      _this->CANbuffer[_this->head].cmsgDLC = canMessage->cmsgDLC;
      for (i=0;i<_this->CANbuffer[_this->head].cmsgDLC;++i)
      {
         _this->CANbuffer[_this->head].cmsgData[i] = canMessage->cmsgData[i];
      }

      _this->head = modulo_inc (_this->head, CANBUFFERLENGTH);
      ++_this->count;
      return (0);
    }
    else
      return (-1);
}

// Flush the ringbuffer
void ringBufS_flush (ringBufS_t *_this, const int clearBuffer)
{
  _this->count  = 0;
  _this->head   = 0;
  _this->tail   = 0;
  if (clearBuffer)
  {
    memset (_this->CANbuffer, 0, sizeof (_this->CANbuffer));
  }
}